"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addJest = void 0;
const addJsExportFile_1 = require("../helpers/addJsExportFile");
const addScriptToPackage_1 = require("../helpers/addScriptToPackage");
const commitWithAuthor_1 = require("../helpers/commitWithAuthor");
const installPackage_1 = require("../helpers/installPackage");
const modifyJsonFile_1 = require("../helpers/modifyJsonFile");
const withStateLogger_1 = require("../helpers/withStateLogger");
const writeAndAddFile_1 = require("../helpers/writeAndAddFile");
exports.addJest = (0, withStateLogger_1.withStateLogger)({ id: "jest" }, (config) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, installPackage_1.installPackage)(config, [
        "jest@29.0.3",
        "@types/jest@29.0.3",
        "ts-jest@29.0.1",
        "ts-node@10.9.1",
        "eslint-plugin-jest@27.0.4",
    ]);
    const jestConfigObject = {
        roots: ["<rootDir>/src"],
        moduleDirectories: ["node_modules", "src"],
        transform: {
            "^.+\\.tsx?$": "ts-jest",
        },
        coverageThreshold: {
            global: {
                statements: 95,
            },
        },
    };
    yield (0, addJsExportFile_1.addCommonJsExportFile)(config, "jest.config.js", jestConfigObject);
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "src/tests/example.test.ts", generateExampleTest());
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, "tsconfig.build.json", tsConfig => (Object.assign(Object.assign({}, tsConfig), { exclude: [...new Set([...(tsConfig.exclude || []), "src/tests", "src/**/*.test.ts"])] })));
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, ".eslintrc.json", eslintRc => (Object.assign(Object.assign({}, eslintRc), { extends: [...new Set([...(eslintRc.extends || []), "plugin:jest/recommended"])], rules: Object.assign(Object.assign({}, (eslintRc.rules || {})), { "jest/expect-expect": "off" }) })));
    yield (0, addScriptToPackage_1.addScriptToPackage)(config, "test", "jest");
    yield (0, commitWithAuthor_1.commitWithAuthor)(config, "Install jest");
}));
const generateExampleTest = () => {
    return `describe("some examples work", () => {
        test("one is one", async () => {
          expect(1).toEqual(1)
        })
      
        test("two is two", async () => {
          expect(2).toEqual(2)
        })
      
        test("throw throws", async () => {
          expect(() => {
            throw new Error("eyyy")
          }).toThrow()
        })
      })
      
      // eslint-disable-next-line jest/no-export
      export {}
    `;
};
