"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupLibrary = void 0;
const addScriptToPackage_1 = require("../helpers/addScriptToPackage");
const commitWithAuthor_1 = require("../helpers/commitWithAuthor");
const installPackage_1 = require("../helpers/installPackage");
const modifyJsonFile_1 = require("../helpers/modifyJsonFile");
const withStateLogger_1 = require("../helpers/withStateLogger");
const writeAndAddFile_1 = require("../helpers/writeAndAddFile");
exports.setupLibrary = (0, withStateLogger_1.withStateLogger)({
    id: "library",
    message: "Configuring project as library",
    failed: "Failed to configure library",
    completed: "Configured project as library",
}, (config) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, installPackage_1.installPackage)(config, ["resolve-tspaths@0.7.5"]);
    yield (0, modifyJsonFile_1.modifyJsonConfig)(config, "package.json", packageJson => (Object.assign(Object.assign({}, packageJson), { files: [...new Set([...(packageJson.files || []), "dist/**"])], keywords: [...new Set([...(packageJson.keywords || []), "library"])], main: "dist/index.js" })));
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "build", "tsc --project tsconfig.build.json && resolve-tspaths -p tsconfig.build.json");
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "prepack", "rm -rf dist && tsc --project tsconfig.build.json && resolve-tspaths -p tsconfig.build.json");
    yield (0, addScriptToPackage_1.appendScriptToPackage)(config, "prepublish", "eslint --cache && tsc --noEmit");
    yield (0, writeAndAddFile_1.writeAndAddFile)(config, "src/index.ts", generateLibraryIndex());
    yield (0, commitWithAuthor_1.commitWithAuthor)(config, "Setup project as library");
}));
const generateLibraryIndex = () => {
    return `export const add = (a: number, b: number) => a + b
    export const subtract = (a: number, b: number) => a - b
    `;
};
